/*
 * FILE --- pid_ctrl.hh
 *
 * PURPOSE --- Header file for a class that implements a Proportional 
 * Integral Derivative (PID) controller. The derivative is obtained as
 * a simple difference between two succesive values of the input to the
 * controller and the integral is calculated using a 4'th order 
 * Runge-Kutta simulation routine. The user defines the PID gains and
 * the sampling time for the controller.
 *
 */

// Prevent Multiple Inclusion
#ifndef PID_CTRL_HH
#define PID_CTRL_HH

// Include Files
#include <stdlib.h>
#include "rk4_sim.hh"

// Local Definitions
#define PID_CTRL_NO_ERROR 0
#define PID_CTRL_INIT_ERROR 1

class PID_CTRL: public RK4_SIM
{
public:
  
  // Constructor
  PID_CTRL();

  // Destructor
  ~PID_CTRL();

  // Specify the sampling time and the initial time
  void init_pid(double _sample_time, double _initial_time = 0.0);
  
  // Set the gains of the PID controller
  void set_gains(double _Kp, double _Ki, double _Kd);
  
  // The main control routine to calculate the controller output 
  // for the given _input, its derivative and integral of the previous inputs
  double control(const double _input);

  int pid_ctrl_err;  // Variable to hold the error condition
  
private:
  double sample_time;  // Sampling time - Neded for calculating the derivative
  double old_input;   // Previous input - Neded for calculating the derivative
  
  // Controller gains
  double Kp;  // Proportional
  double Ki;  // Integral
  double Kd;  // Derivative
  
  // Overload DERIV function of RK4_SIM for the integrator
  inline void DERIV(const double t, const double *x, 
                    const double *u, double *xdot);

  double derivative; // Conveniance variable
};


#endif // PID_CTRL_HH
